# Script version: 4-6-0
# Script date: January 2022
#
# the following actions are performed by this script:
#
#	- add multiple transport connections to an existing node in anynode
#
# to use this script, there are some variables to be defined
#
#	The path to the REST API PowerShell Module needs to be set.
#		- By default, the module is located a folder above the one in which this script is located.
#	The address of anynode has to be defined.
#
# Please make sure to review all possible parameters before
# starting the script.

# Set execution environment
Set-ExecutionPolicy -ExecutionPolicy Unrestricted -Scope Process

# The REST API PowerShell Module's path must be given here. In this case, the file is in the directory above the current one.
Import-Module "./../anynode_rest_api_powershell.psm1"

# Define anynode's Address and Port.
$AnynodeAddress='127.0.0.1'
$AnynodePort=443

# If you have not established a secure connection (HTTPS) to anynode,
# you can use HTTP instead.
# This is not recommended, since your data will NOT be encrypted!
# To activate it anyway, uncomment the following line:
# Disable-CertificateCheck

# Authentication
$Cred=Get-Credential

# Create a list of Transport Connection (Information) to iterate through later.
$TransportConnectionInformation=(
	# First Transport Connection
	[pscustomobject]@{
		SipNode='TE-SYSTEMS Provider'

		DisplayName='Registration 1'

		MatchingCondition=[pscustomobject]@{
			type='ALWAYS'
		}

		AuthenticationEnabled=$true
		Credentials=[pscustomobject]@{
			username='Reg1'
			password='passwordReg1'
		}

		UseAProxyServer=$true
		Proxy=[pscustomobject]@{
			iri='sip:172.18.0.179'
			options=$true
		}

		RegistrationEnabled=$true
		SipDomain='sip:172.18.0.170'
		AddressOfRecord='sip:+4953638195999'
	},
	# Second Transport Connection
	[pscustomobject]@{
		SipNode='TE-SYSTEMS Provider'

		DisplayName='Registration 2'

		MatchingCondition=[pscustomobject]@{
			type='ALWAYS'
		}

		AuthenticationEnabled=$true
		Credentials=[pscustomobject]@{
			username='Reg2'
			password='passwordReg2'
		}

		UseAProxyServer=$true
		Proxy=[pscustomobject]@{
			iri='sip:172.18.0.179'
			options=$true
		}

		RegistrationEnabled=$true
		SipDomain='sip:172.18.0.170'
		AddressOfRecord='sip:+4953638195998'
	}
)

# ----------- Start of script -----------

$Output=''

# Iterate through the previously created list to create Transport Connections.
foreach ($TransportConnection in $TransportConnectionInformation) {
	try {
		# Look for the desired node. ATTENTION: The display name must not be ambiguous!
		$SipNode=(Get-anynodeNodes -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred | Where-Object -Property 'displayName' -Like $TransportConnection.SipNode)
		$Output+="`n"
        $Output+="$SipNode`n"

		# Create the Transport Connection. After this, it will not be added to the node just yet.
		$StandardTransportConnection=Add-anynodeStandardTransportConnection -Address $AnynodeAddress -Port $AnynodePort -DisplayName $TransportConnection.DisplayName -SipNodeId $SipNode.id -AuthenticationEnabled $TransportConnection.AuthenticationEnabled -AuthenticationPolicy $TransportConnection.Credentials -UseAProxyServer $TransportConnection.UseAProxyServer -Proxy $TransportConnection.Proxy -RegistrationEnabled $TransportConnection.RegistrationEnabled -SipDomain $TransportConnection.SipDomain -AddressOfRecord $TransportConnection.AddressOfRecord -Credential $Cred 
		$Output+="$StandardTransportConnection`n"

		# Add the created Transport Connection to the desired node, using the set matching condition.
		$Output+=Add-anynodeNodeTransportConnection -Address $AnynodeAddress -Port $AnynodePort -Node $SipNode.id -TransportConnectionId $StandardTransportConnection.id -MatchingCondition $TransportConnection.MatchingCondition -Credential $Cred
        $Output+="`n"

	} catch {
		$Output+="$_`n"
	}
}


# Commit configuration changes.
$Output+=Push-anynodeConfig -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

echo $Output

Read-Host 'Press Enter to end'
