﻿# Script version: 4-6-0
# Script date: January 2022
#
# the following actions are performed by this script:
#
# 	- Retrieve anynode's dashboard
#	- Find a given registrar
#	- Check if a certain user is registered
#
# to use this script, there are some variables to be defined
#
#	The path to the REST API PowerShell Module needs to be set.
#	- By default, the module is located a folder above the one in which this script is located.
#	The address of anynode has to be defined.
#
# Please make sure to review all possible parameters before
# starting the script.

# Set execution environment
Set-ExecutionPolicy -ExecutionPolicy Unrestricted -Scope Process

# The REST API PowerShell Module's path must be given here. In this case, the file is in the directory above the current one.
Import-Module "./../anynode_rest_api_powershell.psm1"

# Define anynode's Address and Port.
$AnynodeAddress='127.0.0.1'
$AnynodePort=443

# If you have not established a secure connection (HTTPS) to anynode,
# you can use HTTP instead.
# This is not recommended, since your data will NOT be encrypted!
# To activate it anyway, uncomment the following line:
# Disable-CertificateCheck

# Authentication
$Cred=Get-Credential

# Display name of the registrar
$RegistrarNodeName="SIP Phones"

# Value to be checked.
#
# "user": The user name.
# "addressOfRecord": The address of record or dial string with which the user must register.
$Property="user"

# Depending on the $Property that is looked up:
# "user": $Value is the name of the user to be checked.
# "addressOfRecord": $Value is the address of record or $Value is the dial string with which 
# the user must register.
#
# This script will return that a user is registered, even if this $Value matches only a part of 
# the desired $Property.
$Value="User 1"

# ----------- Start of script -----------
try {
# Retrieve a list of SIP nodes from anynode's dashboard.
$Dashboard=Get-anynodeDashboard -Address $AnynodeAddress -Port $AnynodePort -List sipNodes -Credential $Cred

# Check whether retrieving the list of SIP nodes was successful or not.
if ($Dashboard.GetType().Name -match 'Exception') {
    echo "$Dashboard.GetType().Name: $Dashboard"
    return
}

# Check for the authentication to be successful.
if ($Dashboard -match '401') {
    echo $Dashboard
    return
}
else{
    echo "`nAuthentication successfull"    
}

# Filter the list of SIP nodes for the desired registrar.
$RegistrarNode=$Dashboard | ? 'displayName' -EQ $RegistrarNodeName

# Check if the registrar was found.
if ($RegistrarNode -eq $null) {
    echo "Could not find registrar node `"$RegistrarNodeName`"."
    return
}
else{
    echo "Registrar node found"
}

# Check if multiple node entries have been found.
if ($RegistrarNode.GetType().Name -eq 'Object[]') {
    echo "Ambiguous call made. Found multiple registars: $RegistrarNode"
    return
}
else{
    echo "Registrar node is "$RegistrarNode.GetType().Name
}

# Get a list of registered clients of the desired registrar.
if ($RegistrarNode.registeredClientList -ne $null){
    $RegisteredClients=$RegistrarNode.registeredClientList
}
else{
    echo "There are no registered Clients"
}
# Check whether the SIP node found is a registrar or not.
if ($RegisteredClients -eq $null) {
    echo "A SIP node `"$RegistrarNodeName`" has been found, but it is not a registrar. Please provide the name of a registrar node."
    return
}
else{
    echo "$RegistrarNodeName is not null"
}

# Return whether a user is in the registered client list or not.
return ($RegisteredClients | ? -Property $Property -Match $Value) -ne $null
} catch {
    $_.Exception
} finally {
    pause
}