#!/bin/bash

# Version anynode 4.6.0
# January 2022

# This script uses curl, cat and jq.
#  In case curl does not trust your web certificate, this script will not work. Please
#  edit all curl commands after the ---- Start of script ---- part to make it work.

# The following actions are perfomed by this script:
# - get all provider profiles from anynode and look for
#   a match of the chosen display name
# - add the provider node to anynode

# To use this script, there are some variables to be defined.
# Please make sure to review all possible parameters before
# starting the script.

# --- Variables ----

# Define anynode's address
anynode='127.0.0.1'

# Authentication
cred='anadmin:password'

# The display name of the provider template to be used
providerProfileName='babyTel'

# Necessary variable for "babyTel"-Template
clientCredentials='{
  "username":"user",
  "password":"user password"
}'

# Node as JSON object
node=$(cat << END
{
  "activateClientAuthentication":true,
  "credentials":$clientCredentials
}
END
)

# ---- Start of script ----
# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

# Get all provider profiles from anynode and look for a match
# of the display name.
# If you want to use an insecure connection, add the -k argument for curl
allProfiles=$(curl -s -u $cred https://$anynode/api/nodes/providerProfiles/get?version=0)
requestedProfile=$(jq -r ".[] | select(.displayName | contains(\"${providerProfileName}\"))" <<< "${allProfiles}")

# Check whether the call is ambiguous or only one profile was found
profileAmount=$(($(jq -n '[inputs | .] | reduce .[] as $i (0; .+$i)' <<< $(jq -r 'length' <<< $requestedProfile))/2))
if ((profileAmount < 1))
then
  echo "Profile with display name $providerProfileName does not exist. Please note that this call is case sensitive."
  exit
elif ((profileAmount > 1))
then
  echo "Ambiguous call. Profile with display name $providerProfileName exists more than once."

  # Show all found profiles
  jq -c '.displayName' <<< ${requestedProfile}
  exit
fi

# Get the profile ID
profileId=$(jq -r .id <<< "${requestedProfile}")

# Add the provider node
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u $cred -d "$node" https://$anynode/api/nodes/addProvider?version=1\&profile=$profileId
echo

# Commit changes
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u $cred -X POST https://$anynode/api/anynodeConfig/commit?version=0
echo
