#!/bin/bash

# Version anynode 4.6.0
# January 2022

# This script uses curl, cat and jq.
#  In case curl does not trust your web certificate, this script will not work. Please
#  edit all curl commands after the ---- Start of script ---- part to make it work.

# The following actions are performed by this script:
#
# - add multiple transport connections to an existing node in anynode

# to use this script, there are some variables to be defined
#
# Please make sure to review all possible parameters before starting the script.

#---- Variables ----

# Define anynode's address and port.
anynode='127.0.0.1'

# Put in credentials of a user with configuration write access and REST access.
cred='anadmin:password'

# List of transport connections to be added
transportConnectionInformation='[
  {
    "node":"TE-SYSTEMS Provider",
    "displayName":"Registration1",
    "matchingCondition":{
      "type":"ALWAYS"
    },
    "authenticationEnabled":true,
    "credentials":{
      "username":"Reg1",
      "password":"passwordReg1"
    },
    "proxy":{
      "iri":"sip:172.18.0.179",
      "options":true
    },
    "registration":{
      "sipDomain":"sip:172.18.0.170",
      "addressOfRecord":"sip:+4953638195999"
    }
  },
  {
    "node":"TE-SYSTEMS Provider",
    "displayName":"Registration2",
    "matchingCondition":{
      "type":"ALWAYS"
    },
    "authenticationEnabled":true,
    "credentials":{
      "username":"Reg2",
      "password":"passwordReg2"
    },
    "proxy":{
      "iri":"sip:172.18.0.179",
      "options":true
    },
    "registration":{
      "sipDomain":"sip:172.18.0.170",
      "addressOfRecord":"sip:+4953638195998"
    }
  }
]'

# ---- Start of script ----
# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

# Retrieve all nodes to check whether the given "node" exists
# If you want to use an insecure connection, use the -k argument for curl
nodes=$(curl -s -u $cred https://$anynode/api/nodes/get?version=0)

# Iterate over all given transport connections
for key in $(jq -r 'keys | .[]' <<< $transportConnectionInformation); do
  transportConnection=$(jq -r ".[$key]" <<< $transportConnectionInformation)
  displayName=$(jq -r .displayName <<< $transportConnection)

  # Check whether the selected node exists
  nodeName=$(jq -r .node <<< $transportConnection)
  node=$(jq -c ".[] | select(.displayName | contains(\"$nodeName\"))" <<< $nodes)

  if [ -z "$node" ]
  then
    # Selected node does not exist, try the next one
    echo No node found by name $nodeName.
    continue
  fi

  echo Node: $node
  nodeId=$(jq -r .id <<< $node)

  # Add the configured transport connection to anynode
  # If you want to use an insecure connection, use the -k argument for curl
  standardTc=$(curl -s -u $cred -X PUT -d "${transportConnection}" https://$anynode/api/transportConnections/addStandard?version=0\&displayName=$displayName)
  echo Transport Connection: $standardTc
  tcId=$(jq -r .id <<< $standardTc)

  # Add the transport connection to the selected node
  # If you want to use an insecure connection, use the -k argument for curl
  curl -s -u $cred -d "{\"transportConnectionId\":\"$tcId\"}" https://$anynode/api/nodes/transportConnections/add?version=0\&node=$nodeId
  echo
done

# Commit the changes to make them active.
# Add the -k argument in case you want to use an insecure HTTPS-connection.
curl -su $cred -X POST "https://${anynode}/api/anynodeConfig/commit?version=0"
echo