#!/bin/bash

# Version anynode 4.6.0
# January 2022

# This script uses curl, cat and jq.
#  In case curl does not trust your web certificate, this script will not work. Please
#  edit all curl commands after the ---- Start of script ---- part to make it work.

# the following actions are performed by this script:
# - Retrieve anynode's dashboard
# - Find a given registrar
# - Check if a certain user is registered

# to use this script, there are some variables to be defined
#
# The address of anynode has to be defined.
#
# Please make sure to review all possible parameters before
# starting the script.

# ---- Variables ----

# Define anynode's address and port.
anynode='127.0.0.1'

# Put in credentials of a user with configuration write access and REST access.
cred='anadmin:password'

# Display name of the registrar
regName='SIP Phones'

# Value to be checked
# "user": The user name
# "addressOfRecord": The address of record or dial string with which the user
#                    must register
property='user'

# Depending on the $property that is looked up:
# "user": $value is the name of the user to be checked
# "addressOfRecord": $value is the address of record or $value is the dial string
#                    with which the user must register
#
# This script will return that the user is registered, even if this $value
# matches only a part of the desired $property.
value='User 1'

# ---- Start of script ----
# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

# Retrieve a list of SIP nodes from anynode's dashboard
# If you want to use an insecure connection, use the -k argument for curl
sipNodes=$(curl -s -u $cred https://$anynode/api/dashboard/get?version=0\&list=sipNodes)

# Filter the list of SIP nodes for the desired registrar
regNode=$(jq -r ".[] | select(.displayName | contains(\"$regName\"))" <<< $sipNodes)

# Get a list of registered clients of the desired registrar
regClients=$(jq -r .registeredClientList <<< $regNode)

client=$(jq -r ".[] | select(.$property | contains(\"$value\"))" <<< $regClients)

if [ -z $client ]
then
  echo User $value is not registered.
else
  echo User $value is registered.
fi
