#!/bin/bash

# Version anynode 4.6.0
# January 2022
#
# Set certificate
# - Specify a certificate file path
#   - Optionally specify a key file path
#   - Optionally specify a password for the certificate/key
# - Specify which node's certificate is to be set
# - Set the node's certificate

anynode="127.0.0.1:443"
credential="anadmin:password"

node="SIP%20Node"

certificate="./certificate.pfx"

# Optional argument. Use only if your certificate needs a private key.
# If you use this, do not forget to remove the comment '#' further down to import the key
# using curl.
# key="./key.pem"

# Optional argument. Use only if your files require a password.
# password="certificatepassword"

# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

# Import the certificate
# If something does not go as planned, add a -v argument to get more information.
# Add the -k argument in case you want to use an insecure HTTPS-connection.
curl -u $credential -X PUT -F "cert=@${certificate}" "https://${anynode}/api/nodes/certificates/set?version=0&node=${node}&password=${password}&mode=CERTIFICATES_ONLY"
echo

# Import the key
# To use this, remove the comment '#' in front of the next curl and echo commands.
# If something does not go as planned, add a -v argument to get more information.
# Add the -k argument in case you want to use an insecure HTTPS-connection.
#curl -u $credential -X PUT -F "key=@${key}" "https://${anynode}/api/nodes/certificates/set?version=0&node=${node}&password=${password}&mode=PRIVATE_KEY_ONLY"
#echo

# Commit the changes to make them active.
# Add the -k argument in case you want to use an insecure HTTPS-connection.
curl -u $credential -X POST "https://${anynode}/api/anynodeConfig/commit?version=0"
echo